import type {AddViewConfig, FormValues, SchoolClassDetailResponseModel} from '../models';
import {useLoginStore} from '@/stores/login';
import {fetchAuthenticated} from '../retrieveData';


const fetchSchoolClassAddViewConfigMocked = (): AddViewConfig => {
  const config: AddViewConfig = {
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'general__basicSettings',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'firstname',
                    label: 'First name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'lastname',
                    label: 'Last name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'User name',
                    required: true,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputDate',
                  props: {
                    name: 'birthday',
                    label: 'Birthday',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'email',
                    label: 'Primary mail',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputPassword',
                  props: {
                    name: 'password',
                    label: 'Password',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputCheckbox',
                  props: {
                    label: 'Disabled',
                    name: 'disabled',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
          {
            label: 'Account',
            name: 'general__account',
            rows: [
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'accountActivationDate',
                    label: 'Activation date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    name: 'expirationDate',
                    label: 'Expiration date',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
          {
            label: 'UCS@school',
            name: 'ucs@school',
            rows: [
              [
                {
                  type: 'UComboBox',
                  props: {
                    name: 'school',
                    label: 'Primary school',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'School1',
                        value: 'school1',
                      },
                      {
                        label: 'School2',
                        value: 'school2',
                      },
                    ],
                  },
                },
              ],
              [
                {
                  type: 'UMultiSelect',
                  props: {
                    name: 'schoolClasses',
                    label: 'School classes',
                    required: true,
                    access: 'write',
                    options: [],
                  },
                },
              ],
              [
                {
                  type: 'UComboBox',
                  props: {
                    name: 'role',
                    label: 'Role',
                    required: true,
                    access: 'write',
                    options: [
                      {
                        label: 'Role1',
                        value: 'role1',
                      },
                      {
                        label: 'role2',
                        value: 'role2',
                      },
                    ],
                  },
                },
              ],
            ],
          },
        ],
      },
    ],
  };
  console.debug('mock config', config);
  return config;
};
export const fetchSchoolClassAddViewConfig = async(): Promise<AddViewConfig | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchSchoolClassAddViewConfig. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchSchoolClassAddViewConfigMocked());
      }, 1000);
    });
  }
  const loginStore = useLoginStore();
  const answer = await fetchAuthenticated(`/ucsschool/bff-users/v1/pages/addView/settings/${loginStore.username}`);

  return await answer.json();
};

export const createSchoolClass = async(values: FormValues): Promise<boolean> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('createSchoolClass. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(true);
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated('/ucsschool/bff-groups/v1/schoolclasses/', {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  return answer.ok;
};

const fetchSchoolClassMocked = (): SchoolClassDetailResponseModel => {
  const schoolClass: SchoolClassDetailResponseModel = {
    url: '',
    allowedActions: ['save'],
    pages: [
      {
        label: 'General',
        name: 'general',
        fieldsets: [
          {
            label: 'Basic settings',
            name: 'basic',
            rows: [
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'schoolYear',
                    label: 'School year',
                    required: false,
                    access: 'write',
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'displayName',
                    label: 'Display name',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputText',
                  props: {
                    name: 'name',
                    label: 'Name',
                    required: false,
                  },
                },
                {
                  type: 'UInputText',
                  props: {
                    name: 'members',
                    label: 'Users',
                    required: true,
                    access: 'write',
                  },
                },
              ],
              {
                type: 'UInputText',
                props: {
                  name: 'groupType',
                  label: 'Group type',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  name: 'courseType',
                  label: 'Course type',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  label: 'Source',
                  name: 'source',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputText',
                props: {
                  label: 'Record ID',
                  name: 'recordID',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputDate',
                props: {
                  label: 'Scheduled deletion date',
                  name: 'scheduledDeletionDate',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'LMS enabled',
                  name: 'isLmsGroup',
                  required: false,
                  access: 'write',
                },
              },
              {
                type: 'UInputCheckbox',
                props: {
                  label: 'IServ enabled',
                  name: 'isIservGroup',
                  required: false,
                  access: 'write',
                },
              },
            ],
          },
          {
            label: 'Account',
            name: 'general__account',
            rows: [
              [
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Activation date',
                    name: 'activationDate',
                    required: false,
                    access: 'write',
                  },
                },
              ],
              [
                {
                  type: 'UInputDate',
                  props: {
                    label: 'Expiry date',
                    name: 'expiryDate',
                    required: false,
                    access: 'write',
                  },
                },
              ],
            ],
          },
        ],
      },
      {
        label: 'Konto',
        name: 'account',
        fieldsets: [
          {
            label: 'Allgemein',
            name: 'general',
            rows: [
              {
                type: 'UInputDate',
                props: {
                  name: 'creationDate',
                  label: 'Creation date',
                  required: false,
                },
              },
              {
                type: 'UInputDate',
                props: {
                  name: 'lastChange',
                  label: 'Date of last change',
                  required: false,
                },
              },
            ],
          },
          {
            label: 'SAML Einstellungen',
            name: 'SAMLEinstellungen',
            rows: [
              {
                type: 'UMultiSelect',
                props: {
                  name: 'serviceprovider',
                  label: 'SAML service provider',
                  options: [],
                  required: false,
                },
              },
            ],
          },
        ],
      },
    ],
    values: {
      schoolYear: '2020',
      displayName: 'display name',
      name: 'username',
      members: 'members',
      groupType: 'group type',
      courseType: 'course type',
      source: 'source',
      creationDate: '2020-01-01',
      dateOfLastChange: '2020-01-01',
      recordID: 'record id',
      activationDate: '2020-01-01',
      expiryDate: '2020-01-01',
      scheduledDeletionDate:  '2020-01-01',
      isLmsGroup: true,
      isIservGroup: true,
      samlServiceProvider: 'saml service provider',
    },
  };
  console.debug('mock school class', schoolClass);
  return schoolClass;
};

export const fetchSchoolClass = async(schoolClass: string): Promise<SchoolClassDetailResponseModel | null> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('fetchSchoolClass. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve(fetchSchoolClassMocked());
      }, 1000);
    });
  }

  const answer = await fetchAuthenticated(`/ucsschool/bff-groups/v1/schoolclass/detail/${schoolClass}`);
  if (!answer.ok) {
    // FIXME differentiate between 404 (school class does not exist) and other (backend failed in some way)
    // ... and other error cases (e.g. login token expired?)
    return null;
  }
  return await answer.json();
};

export const updateSchoolClass = async(url: string, values: FormValues): Promise<string> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('updateSchoolClass. VITE_USE_REAL_BACKEND===false in DEV mode. Returning mock data');
    // TODO have the fail case in some way too
    return new Promise(resolve => {
      setTimeout(() => {
        resolve('');
      }, 1000);
    });
  }

  const fixedProtocolUrl = url.replace(/https?:/, location.protocol);
  const answer = await fetchAuthenticated(`${fixedProtocolUrl}`, {
    method: 'PATCH',
    headers: {
      'Content-Type': 'application/json',
    },
    body: JSON.stringify(values),
  });

  if (!answer.ok) {
    if (answer.status === 404) {
      return 'dataAccess.updateSchoolClass.error.404';
    }
    if (answer.status >= 500 && answer.status < 600) {
      return 'dataAccess.updateSchoolClass.error.5xx';
    }
    if (answer.status >= 400 && answer.status < 500) {
      return 'dataAccess.updateSchoolClass.error.4xx';
    }
  }
  return '';
};

export const deleteSchoolClass = async(usernames: string[]): Promise<string[]> => {
  if (import.meta.env.VITE_USE_REAL_BACKEND !== 'true') {
    console.debug('deleteSchoolClass. VITE_USE_REAL_BACKEND===false in DEV mode. Doing nothing');
    return new Promise(resolve => {
      setTimeout(() => {
        resolve([usernames[0] as string]);
      }, 1000);
    });
  }

  const fails: string[] = [];
  const promises: Promise<Response>[] = [];
  for (const username of usernames) {
    const promise = fetchAuthenticated(`/ucsschool/bff-groups/v1/schoolclasses/${username}`, {
      method: 'DELETE',
    });
    promise.then(res => {
      if (!res.ok) {
        fails.push(username);
      }
    });
    promises.push(promise);
  }
  await Promise.all(promises);
  return fails;
};
